# HABSKY -- Divide extracted multispec spectra by sky absorption spectra
#          saving original file in Noskyab/
# By Doug Mink, SAO Telescope Data Center
# June 30, 2005

procedure habsky (images)

string	images = ""	{prompt="Input multispec images to correct"}
string	skyfile="hectospec$lib/skyabs.ms.fits"	{prompt="Sky absorption spectrum normalized to continuum=1"}
string	apertures="1-300"	{prompt="List of input apertures or columns/lines"}
string	bands=""	{prompt="List of input bands or lines/bands"}
bool	saveold = yes	{prompt="Save unprocessed file in Noskyab/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file imroot, imnorm, newfile, templist, imin, imsky
	int lroot
	string imout, history, ims, ap, tchar, keypath

	ap = apertures
	ims = images
	vb = verbose
	db = debug
	imsky = skyfile
	db = debug

	if (ims == "") {
	    print ("HABSKY:  Divide extracted multispec spectra by sky absorption spectrum")
	    print ("Usage: habsky <input images>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("abskylist")
	    files (ims, >templist)
	    }
        list = templist
        while (fscan (list,imin) != EOF) {

# Make sure input is a FITS file if apertures are not specified
	    if (ap == "") {
		smatch (imin, ".ms")
		if (smatch.found) {
		    print ("HABSKY: You must specify apertures for file ", imin)
		    next
		    }
		smatch (imin, ".fit")
		if (!smatch.found) {
		    imin = imin // ".fits"
		    if (!access (imin)) {
			print ("HABSKY: ", imin," is not a FITS file")
			next
			}
		    }
		}

# Make sure input is a FITS multispec file if apertures are specified
	    else {
		smatch (imin, ".ms")
		if (!smatch.found) {
		    smatch (imin, ".fits")
		    if (smatch.found) {
			lroot = smatch.start - 1
			imin = substr (imin, 1, lroot) // ".ms.fits"
			}
		    else {
			imin = imin // ".ms.fits"
			}
		    if (!access (imin)) {
			print ("HABSKY: ", imin," does not exist")
			next
			}
		    }
		else {
		    smatch (imin, ".fits")
		    if (!smatch.found) {
			imin = imin // ".fits"
			}
		    }
		}

# Do not remove sky absorption twice
	    imgetp.default = "0"
	    imgetp (imin, "HABSKY")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HABSKY: ", imin, " has already been processed")
		    print ("HABSKY = ", imgetp.value)
		    }
		next
		}

# Copy original file to Noskyab directory
	    if (saveold) {
		if (!access ("Noskyab/")) {
		    mkdir ("Noskyab")
		    }
		newfile = "Noskyab/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		imcopy.verbose = db
		imcopy (imin, "Noskyab")
		if (vb) {
		    print ("HABSKY: Original ", imin, " => Noskyab/")
		    }
		}

# Divide out normalization spectra
	    imout = mktemp ("abskyout") // ".ms.fits"
	    sarith.w1 = INDEF
	    sarith.w2 = INDEF
	    sarith.apertures = ap
	    sarith.bands = bands
	    sarith.beams = ""
	    sarith.apmodulus = 0
	    sarith.reverse = no
	    sarith.ignoreaps = yes
	    if (ap == "") {
		sarith.format = "onedspec"
		}
	    else {
		sarith.format = "multispec"
		}
	    sarith.renumber = no
	    sarith.offset = 0
	    sarith.clobber = yes
	    sarith.merge = no
	    sarith.rebin = yes
	    sarith.errval = 0.
	    sarith.verbose = db
	    sarith (imin, "/", imsky, imout)

# Fix APID keywords
	    if (ap != "") {
		imgetp.default = "0"
	 	imgetp (imin, "APID1")
		if (imgetp.value != "0") {
		    keypath = "@" // osfn("hectospec$lib/") // "apnumkeys"
		    print ("delhead ",imout," ",keypath)
		    dhead (imout,keypath)
		    keypath = "@" // osfn("hectospec$lib/") // "apidkeys"
		    print ("cphead ",imin," ",imout," ",keypath)
		    chead (imin,imout,keypath)
		    }
		}

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " " // imsky // " divided out"
	    hedit (imout, "HABSKY", history, add+, verify-, show-)
	    if (vb) {
		print ("HABSKY: ",history)
		}

# Rename output file to same as input file (which is saved if saveold=yes)
	    imdelete (imin)
	    imcopy (imout, imin)

	    if (deltemp) {
		imdelete (imout)
		}
	    }

	if (tchar != '@' && deltemp) {
	    delete (templist)
	    }
end

# Aug  6 2004	New program
# Aug 26 2004	Make it work on 1D files, too
# Aug 27 2004	Change logging keyword to HABSKY from SKYAB
# Aug 31 2004	Don't process image if HABSKY keyword is in header
# Aug 31 2004	Read next image if there is a problem instead of stopping
# Sep  2 2004	Process multiple input images
# Oct 13 2004	Check for .ms, not .ms.fit
# Oct 14 2004	Write sarith output to new file and rename it
# Oct 14 2004	Replace sarith-written APNUMnnn with APIDnnn from input file
# Oct 18 2004	Print HABSKY value if already processed

# Jan 11 2005	Improve input filename options
# Feb 23 2005	Drop keyword copying; use multispec version of sky spectrum
# Apr 14 2005	Really delete temporary files if requested
# Apr 15 2005	Fix APID keywords if they are deleted by IRAF
# Jun  8 2005	Delete APNUM keywords, copy APID keywords, allow *.ms
# Jun 30 2005	Change external cphead,delhead calls to wcstools.chead,dhead
