# HLIN1 -- Dispersion correct and linearize spectrum, placing input in Nodisp/
# By Doug Mink, SAO Telescope Date Center
# June 30, 2005

procedure hlin1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
real wl1=3700.0		{prompt="Starting wavelength for output spectra (A)"}
real wl2=9150.0		{prompt="Ending wavelength for output spectra (A)"}
real dwl=1.21	{prompt="Delta wavelength per pixel for output spectra (A)"}
int npix = INDEF	{prompt="Number of pixels per output spectra"}
bool saveold = yes	{prompt="Save original files in Nodisp/ (y or n)"}
bool verbose = no	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Too much info from sumspec (y or n)"}

begin
	bool vb, db, save
	file imin, imlin, imdisp, imroot, imcal, newfile, templist
	string history
	int lroot, lref, nap
	real pixshift

	imin = image
	vb = verbose
	db = debug
	if (db) {
	    vb = yes
	    }
	save = saveold

	if (imin == "") {
	    print ("HLIN1:  Linearize file of spectra")
	    print ("Usage: hlin1 <input image>")
	    return
	    }

# Make output file name and check to make sure input is multispec
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imin = imroot // ".ms.fits"
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		}
	    }
	if (!access (imin)) {
	    print ("HLIN1: ", imin," does not exist")
	    return
	    }

# Do not linearize image twice
	imgetp.default = "0"
	imgetp (imin, "HLIN")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HLIN1: ", imin, " has already been linearized")
		print ("HLIN = ", imgetp.value)
		}
	    return
	    }

# Make output file name
	imlin = imroot // ".lin.ms.fits"

# Get number ofspectra in input file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)

# Read pixel shift from image header
	imgetp.default = "0"
	imgetp (imin, "PIXSHIFT")
	pixshift = real (imgetp.value)
	if (vb) {
	    if (pixshift == 0.0) {
		print ("HLIN1: ", nap," apertures")
		}
	    else {
		print ("HLIN1: ", nap,"apertures shifted ", pixshift, "pixels")
		}
	    }

# Rebin spectra identically
	rvsao.sumspec.spectra = imin
	rvsao.sumspec.compfile = imlin
	rvsao.sumspec.specnum = "1-" // nap
	rvsao.sumspec.specband = 0
	rvsao.sumspec.specdir = ""
	rvsao.sumspec.compname = ""
	rvsao.sumspec.compdir = ""
	rvsao.sumspec.nspec = nap
	rvsao.sumspec.save_names = no
	rvsao.sumspec.normin = 0.
	rvsao.sumspec.fixbad = "no"
	rvsao.sumspec.cont_remove = "no"
	rvsao.sumspec.reject = no
	rvsao.sumspec.abs_reject = 100.
	rvsao.sumspec.em_reject = 2.
	rvsao.sumspec.contout = "no"
	rvsao.sumspec.cont_plot = no
	rvsao.sumspec.cont_add = 0.
	rvsao.sumspec.spec_smooth = 0
	rvsao.sumspec.st_lambda = wl1
	rvsao.sumspec.end_lambda = wl2
	rvsao.sumspec.pix_lambda = dwl
	rvsao.sumspec.npts = npix
	rvsao.sumspec.complog = no
#	rvsao.sumspec.interp_mode = "sums"
	rvsao.sumspec.interp_mode = "spline3"
	rvsao.sumspec.normout = 0.
	rvsao.sumspec.spec_plot = no
	rvsao.sumspec.spec_int = no
	rvsao.sumspec.comp_plot = vb
	rvsao.sumspec.comp_int = no
	rvsao.sumspec.velcomp = INDEF
	rvsao.sumspec.zcomp = INDEF
	rvsao.sumspec.svel_corr = "none"
	rvsao.sumspec.nsmooth = 0
	rvsao.sumspec.device = "stdgraph"
	rvsao.sumspec.plotter = "stdplot"
	rvsao.sumspec.logfiles = "STDOUT,displin.log"
	rvsao.sumspec.nsum = 1
	rvsao.sumspec.debug = db
	rvsao.sumspec.cursor = ""
	rvsao.sumspec (imin, imlin)

# Move original file to Nolin directory, if requested
	if (save) {
	    if (!access ("Nolin/")) {
		mkdir ("Nolin")
		}
	    newfile = "Nolin/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Nolin")
	    if (vb) {
		print ("HLIN1: Nonlinear ", imin, " => Nolin/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imlin, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " image linearized"
	if (pixshift != 0.0) {
	    history = history // " shifted " // pixshift
	    }
        hedit (imin, "HLIN", history, add+, verify-, show=vb)
        hedit (imin, "PIXSHIFT", delete+, verify-, show=vb)

end
# Jun 10 2004	New task
# Sep  2 2004	Run on multiple files via list argument or file
# Sep  2 2004	Add keyword HLIN; do not linearize image when it already exists
# Oct 13 2004	Drop extra closing brace
# Oct 18 2004	If already processed, print HLIN value

# Jan 12 2005	Read number of apertures from image header
# Mar  7 2005	Separate options to save original and dispersed files
# Mar 21 2005	Do not change deltemp if reading from file list
# May 11 2005	Set deltemp=no and verbose=yes if debug=yes
# May 16 2005	Separate linearization for single file into its own task
# May 16 2005	Drop deltemp; it's used by hlin
# May 19 2005	Split off dispersion correction to HDISP1
# May 20 2005	Set sumspec.pixshift from PIXSHIFT in image header
# May 24 2005	Add shift to HLIN history and delete PIXSHIFT keyword
# May 25 2005	Drop sumspec.pixshift; it comes from the image header
# Jun  6 2005	Change interpolation from spline3 to sums
# Jun 24 2005	Change interpolation back to spline3
# Jun 30 2005	Load wcstools and rvsao in hectospec.cl
