# HMERGE - Combine 4 amps into one Hectospec image or two Hectochelle images
# March 17, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hmerge (images)

string	images=""	{prompt="Image file(s) to merge"}

bool	perchip=yes	{prompt="Merge amps to one output file per CCD (y or n)"}
bool	saveold=yes	{prompt="Save unmerged files in Unmerged/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, temp4, im4, imin, templist, imout
file	imfile, newfile, imex0
string	amp1, amp2, amp3, amp4, arg1, imx, ims, history, tchar
string	gain1, gain2, gain3, gain4
bool	dellist, vb, perc

	vb = verbose
	imx = ".fits"
	ims = images
	perc = perchip

	if (ims == "") {
	    print ("HMERGE: Combine 4 amps on chips from 1 Hectospec image or 2 Hectochelle images")
	    print ("Usage: hmerge <input image with 4 extensions>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
            templist = mktemp ("mergelist")
            files (ims, >templist)
	    }
        list = templist
        while (fscan (list,imin) != EOF) {

	
# Do not merge amplifiers twice
	    imgetp.default = "0"
	    imex0 = imin // "[0]"
	    imgetp (imex0, "HMERGE")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HMERGE: ", imin, " has already been merged")
		    print ("HMERGE = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		}
	    if (vb) {
		print ("HMERGE: image root is ",imroot)
		}

#  Split out one image per amplifier
	    temp1 = mktemp ("hamp1") // imx
	    amp1 = imroot // "[1]"
	    imcopy (amp1, temp1)
	    temp2 = mktemp ("hamp2") // imx
	    amp2 = imroot // "[2]"
	    imcopy (amp2, temp2)
	    temp3 = mktemp ("hamp3") // imx
	    amp3 = imroot // "[3]"
	    imcopy (amp3, temp3)
	    temp4 = mktemp ("hamp4") // imx
	    amp4 = imroot // "[4]"
	    imcopy (amp4, temp4)

# Get gain correction terms for output header(s)
	    imgetp.default = "1.0"
	    imgetp (amp1, "GAINCORR")
	    gain1 = imgetp.value
	    imgetp.default = "1.0"
	    imgetp (amp2, "GAINCORR")
	    gain2 = imgetp.value
	    imgetp.default = "1.0"
	    imgetp (amp3, "GAINCORR")
	    gain3 = imgetp.value
	    imgetp.default = "1.0"
	    imgetp (amp4, "GAINCORR")
	    gain4 = imgetp.value

# Move original file to Unflat directory
	    if (saveold) {
		if (!access ("Unmerged/")) {
		    mkdir ("Unmerged")
		    }
		newfile = "Unmerged/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		move (imin, "Unmerged")
		if (vb) {
		    print ("HMERGE: Original ", imin, " => Unmerged/")
		    }
		}
	    else {
		delete (imin)
		}
            gettime ()

#  Join two amplifiers per chip into two images
	    if (perc) {
		arg1 = temp1 // "," // temp2 // "[-*,*]"
		imout = imroot // "_chip1" // imx
		imjoin (arg1, imout, 1)

# Add gain corrections from each amplifier
		hedit (imout, "GAINCOR1", gain1, add+, verify-)
		hedit (imout, "GAINCOR2", gain2, add+, verify-)

# Make history line including time for preamp correction
		hedit (imout, "GAINCORR", "", delete+, verify-)
		history = gettime.date // " " // gettime.time // " Amplifiers 1 and 2 merged into one image"
		hedit (imout, "HMERGE", history, add+, verify-)

#  Join second chip amplifiers into a second file
		arg1 = temp3 // "," // temp4 // "[-*,*]"
		imout = imroot // "_chip2" // imx
		imjoin (arg1, imout, 1)

# Add gain corrections from each amplifier
		hedit (imout, "GAINCORR", "", delete+, verify-)
		hedit (imout, "GAINCOR3", gain3, add+, verify-)
		hedit (imout, "GAINCOR4", gain4, add+, verify-)

# Make history line including time for preamp correction
		history = gettime.date // " " // gettime.time // " Amplifiers 3 and 4 merged into one image"
		hedit (imout, "HMERGE", history, add+, verify-)
		}

#  Join two chips into a single image
	    else {
		arg1 = temp1 // "," // temp2 // "[-*,*]," // temp3 // "," // temp4 // "[-*,*]"
		imjoin (arg1, imin, 1)

# Add gain corrections from each amplifier
		hedit (imin, "GAINCORR", "", delete+, verify-)
		hedit (imin, "GAINCOR1", gain1, add+, verify-)
		hedit (imin, "GAINCOR2", gain2, add+, verify-)
		hedit (imin, "GAINCOR3", gain3, add+, verify-)
		hedit (imin, "GAINCOR4", gain4, add+, verify-)

# Make history line including time for preamp correction
		history = gettime.date // " " // gettime.time // " 4 amplifiers merged into one image"
		hedit (imin, "HMERGE", history, add+, verify-)
		}

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		imdelete (temp3)
		imdelete (temp4)
		}
	    }
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Jun 22 2004	Program to make a images from four amplifier image extensions
# Sep  2 2004	Do not process image if HMERGE keyword already exists
# Sep  2 2004	Merge multiple files
# Oct 13 2004	Check for HMERGE in 0th extension (only extension if processed)
# Oct 18 2004	Print HMERGE value if amplifiers are already merged
# Dec 21 2004	Add option to merge amplifiers on each chip and write two files

# Jan 13 2005	Change per-chip suffixes from _1 and _2 to _chip1 and _chip2
# Mar 17 2005	Do not reset deltemp if input from file list
