# HNORM -- Divide extracted multispec spectra by pixel normalization file
#          saving original file in Nonorm/
# By Doug Mink, SAO Telescope Data Center
# April 11, 2005

procedure hnorm (images)

string	images = ""	{prompt="Input multispec image(s) to correct"}
string	normfile=""	{prompt="Pixel to pixel normalization file from makenorm"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed file in Nonorm/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imin, imout, imroot, imnorm, newfile, templist
	int lroot
	string history, tchar

	ims = images
	vb = verbose
	imnorm = normfile

	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("normlist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name and check to make sure input is multispec
	    smatch (imin, ".ms")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imout = substr (imin, 1, lroot) // ".norm.ms.fits"
		}
	    else {
		print ("HNORM: ", imin," is not a multispec file")
		return
		}

# Make sure filename ends in .fits
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}

# Do not normalize image twice
	    imgetp.default = "0"
	    imgetp (imin, "HNORM")
	    if (imgetp.value == "0") {
		imgetp (imin, "PIXNORM")
		}
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HNORM: ", imin, " has already been normalized")
		    print ("HNORM = ", imgetp.value)
		    }
		next
		}

# Divide out normalization spectra
	    imarith (imin, "/", imnorm, imout)

# Move original file to Nonorm directory
	    if (saveold) {
		if (!access ("Nonorm/")) {
		    mkdir ("Nonorm")
		    }
		newfile = "Nonorm/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		move (imin, "Nonorm")
		if (vb) {
		    print ("HNORM1: Original ", imin, " => Nonorm/")
		    }
		}
	    else {
		delete (imin)
		}

# Rename result file to original file name
	    rename (imout, imin)

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " pixels normalized by " // imnorm
	    hedit (imin, "HNORM", history, add+, verify-)
	    }
	if (deltemp) {
	    delete (templist)
	    }

end
# Jun 16 2004	Change name from HNORM1 to HNORM
# Sep  2 2004	Change history keyword from PIXNORM to HNORM
# Sep  2 2004	Run on multiple image files at once
# Sep  2 2004	Test history keyword to avoid rerunning program on file
# Oct 18 2004	Print value of HNORM/PIXNORM if image has already been normalized

# Apr 11 2005	Change name of temporary file list to normlist*
