# HTRAN1 -- Correct single multispec image for fiber throughput differences
# By Doug Mink, SAO/TDC
# March 21, 2005

procedure htran1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
string	skycorr=""	{prompt="Throughput correction image from maketran"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed files to Notran/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Log each aperture (y or n)"}

begin
	bool vb, db
	file imin, imout, imcorr, imroot, newfile
	string history, order, function, ims, tchar
	int nap, npix, lroot

	imin = image
	imcorr = skycorr
	vb = verbose
	db = debug

# Make sure filename ends in .ms.fits
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".corr.ms.fits"
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		imout = substr (imin, 1, lroot) // ".corr.ms.fits"
		}
	    else {
		imout = imin // ".corr.ms.fits"
		imin = imin // ".ms.fits"
		}
	    }

	if (!access (imin)) {
	    print ("HTRAN1: ", imin," does not exist")
	    return
	    }

# Check to see whether fiber color throughput correction has been done
	imgetp.default = "0"
	imgetp (imin, "HTRAN")
	if (imgetp.value == "0") {
	    imgetp (imin, "FIBRTRAN")
	    }
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HTRAN1: ", imin, " has been throughput corrected")
		print (" HTRAN = ", imgetp.value)
		}
	    return
	    }
	else if (vb) {
	    print ("HTRAN1: ", imin," -> ",imout)
	    }

# Make sure throughput correction file is set
	if (imcorr == "") {
	    if (vb) {
		print ("HTRAN1: ", imin, " has been no throughput correction file")
		}
	    return
	    }

# Make sure throughput correction filename ends in .ms.fits
	smatch (imcorr, ".ms")
	if (smatch.found) {
	    smatch (imcorr, ".fits")
	    if (!smatch.found) {
		imcorr = imcorr // ".fits"
		}
	    }
	else {
	    smatch (imcorr, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imcorr = substr (imcorr, 1, lroot) // ".ms.fits"
		}
	    else {
		imcorr = imcorr // ".ms.fits"
		}
	    }

# Make sure that throughput correction file exists
	if (!access (imcorr)) {
	    if (vb) {
		print ("HTRAN1: ", imcorr, " does not exist")
		}
	    return
	    }

# Get number of apertures and pixels from header of input multispec image
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	if (vb) {
	    print ("HTRAN1: ", nap," apertures of ",npix," pixels each")
	    }

# Correct all spectra for fiber color throughput differences
	imarith (imin, "/", imcorr, imout, verbose=yes)

# Move original file to Notran directory
	if (saveold) {
	    if (!access ("Notran/")) {
		mkdir ("Notran")
		}
	    newfile = "Notran/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Notran")
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	imrename (imout, imin)

# Add history lines including time about this throughput correction file
	gettime ()
	imgetp (imcorr, "TRANORD")
	order = imgetp.value
	imgetp (imcorr, "TRANFUNC")
	function = imgetp.value
	gettime ()
	history = gettime.date // " " // gettime.time // " fiber throughput corrected: " // order // " order " // function
	hedit (imin, "HTRAN", history, add+, verify-)
	hedit (imin, "FIBRFILE", imcorr, add+, verify-)
end

# Mar  9 2005	New task moved from HTRAN
# Mar 21 2005	Fix bug checking throughput correction image for .fits
